from .alerts import AlertStubMixin
from .analysis import AnalysisStubMixin
from .identities import IdentityStubMixin
from .plugins import PluginStubMixin
from .streams import StreamStubMixin
from .zone_statuses import ZoneStatusStubMixin
from .zones import ZoneStubMixin
from .storage import StorageStubMixin
from .alarms import ZoneAlarmStubMixin
from .process_image import ProcessImageStubMixIn
from .encodings import EncodingStubMixIn
from .premises import PremisesStubMixin
from .users import UserStubMixin
from .licenses import LicenseStubMixIn
from .base_stub import BaseStub
